@@ -12,45 +12,81 @@ CompLik2 <- function(copula,bivariate, coordx, coordy ,coordz,coordt,coordx_dyn,
   {
 
 
-comploglik2 <- function(param,colidx,rowidx, corrmodel, coords,data1,data2,fixed, fan, n, namescorr, 
-                              namesnuis,namesparam,namesaniso,weigthed,X,MM,aniso,type_cop,cond_pair)
-      {
-        names(param) <- namesparam
-        param <- c(param, fixed)
-        #print(param)
-        paramcorr <- as.numeric(param[namescorr])
-        nuisance <- param[namesnuis]
-        sel=substr(names(nuisance),1,4)=="mean"
-        if(is.null(MM)){ mm=as.numeric(nuisance[sel]) ### linear mean
-                         Mean=c(X%*%mm)
-                       }
-        else           Mean=MM                     ### fixed mean
-        other_nuis=as.numeric(nuisance[!sel])   ## or nuis parameters (nugget sill skew df)         
-      #  print(other_nuis);print(fan)
-      #  print(paramcorr)
-        if(aniso){
-            anisopar<-param[namesaniso]
-            coords1=GeoAniso(coords, anisopars=anisopar)
-          c1=c(t(coords1[colidx,]));c2=c(t(coords1[rowidx,]))
-          result=dotCall64::.C64(as.character(fan),
-         SIGNATURE = c("integer","double","double","double","double", "integer","integer","double","integer","double","double","double","double","integer","integer"),  
-                        corrmodel,c1,c2,data1, data2, n[colidx],n[rowidx],paramcorr,weigthed, res= dotCall64::vector_dc("double",1),Mean[colidx], Mean[rowidx], other_nuis,
-                        type_cop,cond_pair,
-          INTENT =    c(rep("r",9),"w",rep("r",5)),
-             PACKAGE='GeoModels', VERBOSE = 0, NAOK = TRUE)$res
-
-
+comploglik2 <- function(param, colidx, rowidx, corrmodel, coords, data1, data2, fixed, fan, n, namescorr, 
+                              namesnuis, namesparam, namesaniso, weigthed, X, MM, aniso, type_cop, cond_pair) {
+    
+    # Assegnazione nomi ed unione parametri (più efficiente)
+    names(param) <- namesparam
+    param <- c(param, fixed)
+    
+    # Estrazione parametri (evitare conversioni multiple)
+    paramcorr <- param[namescorr]
+    nuisance <- param[namesnuis]
+    
+    # Identificazione parametri mean (più veloce di substr)
+    sel <- startsWith(names(nuisance), "mean")
+    
+    # Calcolo Mean ottimizzato
+    if (is.null(MM)) {
+        mm <- nuisance[sel]
+        # Usa tcrossprod per maggiore efficienza se X è grande
+        if (ncol(X) > 10) {
+            Mean <- as.vector(tcrossprod(mm, X))
+        } else {
+            Mean <- as.vector(X %*% mm)
         }
-         else
-         {      
-         result=dotCall64::.C64(as.character(fan),
-         SIGNATURE = c("integer","double","double", "integer","integer","double","integer","double","double","double","double","integer","integer"),  
-                        corrmodel,data1, data2, n[colidx],n[rowidx],paramcorr,weigthed, res= dotCall64::vector_dc("double",1),Mean[colidx], Mean[rowidx], other_nuis,type_cop,cond_pair,
-          INTENT =    c(rep("r",7),"w",rep("r",5)),
-             PACKAGE='GeoModels', VERBOSE = 0, NAOK = TRUE)$res
-       }
-return(-result)
-      }
+    } else {
+        Mean <- MM
+    }
+    
+    # Estrazione altri parametri nuisance
+    other_nuis <- nuisance[!sel]
+    
+    # Branch anisotropia ottimizzato
+    if (aniso) {
+        # Pre-calcolo coordinate anisotrope
+        coords1 <- GeoAniso(coords, anisopars = param[namesaniso])
+        
+        # Estrazione coordinate ottimizzata (evita transposizioni multiple)
+        c1 <- as.vector(t(coords1[colidx, , drop = FALSE]))
+        c2 <- as.vector(t(coords1[rowidx, , drop = FALSE]))
+        
+        # Chiamata C ottimizzata
+        result <- dotCall64::.C64(
+            as.character(fan),
+            SIGNATURE = c("integer", "double", "double", "double", "double", 
+                         "integer", "integer", "double", "integer", "double", 
+                         "double", "double", "double", "integer", "integer"),  
+            corrmodel, c1, c2, data1, data2, 
+            n[colidx], n[rowidx], paramcorr, weigthed, 
+            res = dotCall64::vector_dc("double", 1),
+            Mean[colidx], Mean[rowidx], other_nuis, type_cop, cond_pair,
+            INTENT = c(rep("r", 9), "w", rep("r", 5)),
+            PACKAGE = 'GeoModels', 
+            VERBOSE = 0, 
+            NAOK = TRUE
+        )$res
+        
+    } else {
+        # Chiamata C senza anisotropia
+        result <- dotCall64::.C64(
+            as.character(fan),
+            SIGNATURE = c("integer", "double", "double", "integer", "integer", 
+                         "double", "integer", "double", "double", "double", 
+                         "double", "integer", "integer"),  
+            corrmodel, data1, data2, 
+            n[colidx], n[rowidx], paramcorr, weigthed, 
+            res = dotCall64::vector_dc("double", 1),
+            Mean[colidx], Mean[rowidx], other_nuis, type_cop, cond_pair,
+            INTENT = c(rep("r", 7), "w", rep("r", 5)),
+            PACKAGE = 'GeoModels', 
+            VERBOSE = 0, 
+            NAOK = TRUE
+        )$res
+    }
+    
+    return(-result)
+}
 ##################################################################
 ##################################################################
 comploglik_biv2 <- function(param,colidx,rowidx, corrmodel, coords,data1,data2,fixed, fan, n, 
